from demoparser.bitbuffer cimport Bitbuffer

cdef class Decoder:
    cdef Bitbuffer buf
    cdef dict fprop
    cdef object prop
    cdef long flags

    # Methods
    cpdef object decode(self)
    cpdef long decode_int(self)
    cpdef float decode_float(self)
    cpdef dict decode_vector(self)
    cpdef dict decode_vector_xy(self)
    cpdef str decode_string(self)
    cpdef list decode_array(self)
    cpdef float decode_special_float(self)

cdef enum PropTypes:
    DPT_Int = 0
    DPT_Float = 1
    DPT_Vector = 2
    DPT_VectorXY = 3
    DPT_String = 4
    DPT_Array = 5
    DPT_DataTable = 6
    DPT_Int64 = 7
    DT_MAX_STRING_BITS = 9

cdef enum PropFlags:
    SPROP_UNSIGNED = (1 << 0)
    SPROP_COORD = (1 << 1)
    SPROP_NOSCALE = (1 << 2)
    SPROP_ROUNDDOWN = (1 << 3)
    SPROP_ROUNDUP = (1 << 4)
    SPROP_NORMAL = (1 << 5)
    SPROP_EXCLUDE = (1 << 6)
    SPROP_XYZE = (1 << 7)
    SPROP_INSIDEARRAY = (1 << 8)
    SPROP_PROXY_ALWAYS_YES = (1 << 9)
    SPROP_IS_A_VECTOR_ELEM = (1 << 10)
    SPROP_COLLAPSIBLE = (1 << 11)
    SPROP_COORD_MP = (1 << 12)
    SPROP_COORD_MP_LOWPRECISION = (1 << 13)
    SPROP_COORD_MP_INTEGRAL = (1 << 14)
    SPROP_CELL_COORD = (1 << 15)
    SPROP_CELL_COORD_LOWPRECISION = (1 << 16)
    SPROP_CELL_COORD_INTEGRAL = (1 << 17)
    SPROP_CHANGES_OFTEN = (1 << 18)
    SPROP_VARINT = (1 << 19)



from suitcase.structure import Structure
from suitcase.fields import FieldArray
from suitcase.fields import Magic
from suitcase.fields import SubstructureField
from suitcase.fields import ULInt32
from suitcase.fields import UBInt32
from suitcase.fields import UBInt64
from suitcase.fields import UBInt8
from suitcase.fields import ULInt8

from demoparser import consts
from demoparser.fields import FixedLengthString
from demoparser.fields import SLFloat32
from demoparser.fields import UBInt32Sequence


class DemoHeader(Structure):
    r"""1072 Byte header for .DEM file.

    This header has the following format:

    +-----------+---------------------------------------+
    | Byte      | Description                           |
    +===========+=======================================+
    | 0-7       | Fixed string 'HL2DEMO\0'.             |
    +-----------+---------------------------------------+
    | 8-11      | Demo file protocol version.           |
    +-----------+---------------------------------------+
    | 12-15     | Network protocol version.             |
    +-----------+---------------------------------------+
    | 16-275    | Server name.                          |
    +-----------+---------------------------------------+
    | 276-535   | Name of client who recorded the demo. |
    +-----------+---------------------------------------+
    | 536-795   | Map name.                             |
    +-----------+---------------------------------------+
    | 796-1055  | Game directory.                       |
    +-----------+---------------------------------------+
    | 1056-1059 | Playback time in seconds.             |
    +-----------+---------------------------------------+
    | 1060-1063 | Number of ticks in demo.              |
    +-----------+---------------------------------------+
    | 1064-1067 | Number of frames in demo.             |
    +-----------+---------------------------------------+
    | 1068-1071 | Length of signon data.                |
    +-----------+---------------------------------------+
    """
    header = Magic(b'HL2DEMO\x00')
    demo_protocol = ULInt32()
    network_protocol = ULInt32()
    server_name = FixedLengthString(consts.MAX_PATH)
    client_name = FixedLengthString(consts.MAX_PATH)
    map_name = FixedLengthString(consts.MAX_PATH)
    game_directory = FixedLengthString(consts.MAX_PATH)
    playback_time = SLFloat32()
    ticks = ULInt32()
    frames = ULInt32()
    signon_length = ULInt32()


class CommandHeader(Structure):
    """Header for each command packet.

    .. _header-format:

    The header has the following format:

    +------+--------------+
    | Byte | Description  |
    +======+==============+
    | 0    | Command ID   |
    +------+--------------+
    | 1-4  | Current tick |
    +------+--------------+
    | 5    | Player ID    |
    +------+--------------+
    """
    command = ULInt8()
    tick = ULInt32()
    player = ULInt8()


class QAngle(Structure):
    pitch = SLFloat32()
    yaw = SLFloat32()
    roll = SLFloat32()


class Vector(Structure):
    x = SLFloat32()
    y = SLFloat32()
    z = SLFloat32()


class OriginViewAngles(Structure):
    view_origin = SubstructureField(Vector)
    view_angles = SubstructureField(QAngle)
    local_view_angles = SubstructureField(QAngle)


class SplitCommandInfo(Structure):
    flags = ULInt32()
    original = SubstructureField(OriginViewAngles)
    resampled = SubstructureField(OriginViewAngles)


class CommandInfo(Structure):
    commands = FieldArray(SplitCommandInfo)


class UserInfo(Structure):
    """Player data.

    This structure has the following format:

    +---------+---------------------------------------+
    | Byte    | Description                           |
    +=========+=======================================+
    | 0-7     | Version. Same for all players.        |
    +---------+---------------------------------------+
    | 8-15    | xuid. Some sort of unique ID.         |
    +---------+---------------------------------------+
    | 15-142  | Player name.                          |
    +---------+---------------------------------------+
    | 143-146 | Local server user ID.                 |
    +---------+---------------------------------------+
    | 147-179 | GUID                                  |
    +---------+---------------------------------------+
    | 180-183 | Friend's ID.                          |
    +---------+---------------------------------------+
    | 184-312 | Friend's Name.                        |
    +---------+---------------------------------------+
    | 313     | Is player a bot?                      |
    +---------+---------------------------------------+
    | 314     | Is player an HLTV proxy?              |
    +---------+---------------------------------------+
    | 314-329 | Custom files CRC.                     |
    +---------+---------------------------------------+
    | 330     | Numbre of files downloaded by server. |
    +---------+---------------------------------------+
    | 331-335 | Entity index.                         |
    +---------+---------------------------------------+
    | 336-340 | No idea.                              |
    +---------+---------------------------------------+
    """
    version = UBInt64()
    xuid = UBInt64()
    name = FixedLengthString(consts.MAX_PLAYER_NAME_LENGTH)
    user_id = UBInt32()
    guid = FixedLengthString(consts.SIGNED_GUID_LEN)
    friends_id = UBInt32()
    friends_name = FixedLengthString(consts.MAX_PLAYER_NAME_LENGTH)
    fake_player = UBInt8()
    is_hltv = UBInt8()
    custom_files = UBInt32Sequence(consts.MAX_CUSTOM_FILES)
    files_downloaded = UBInt8()
    entity_id = UBInt32()
    tbd = UBInt32()



from demoparser.bitbuffer cimport Bitbuffer
from demoparser.bytebuffer import Bytebuffer
from libc cimport math

from collections import defaultdict
from collections import deque
from collections import OrderedDict

from demoparser import consts
from demoparser.protobufs import netmessages_pb2
from demoparser.protobufs import cstrike15_usermessages_pb2 as um
from demoparser.entities import EntityList
from demoparser.props cimport PropFlags
from demoparser.props cimport PropTypes
from demoparser.structures import UserInfo
from demoparser.structures import DemoHeader

from demoparser.util cimport parse_entity_update


cdef class CommandError(Exception):
    pass


cdef class ClassNotFoundError(Exception):
    pass


cdef bint _key_sort(dict item):
    if item.get('collapsible', True) is False:
        return 0
    return 1


cdef enum DemoCommand:
    SIGNON = 1
    PACKET = 2
    SYNCTICK = 3
    CONSOLECMD = 4
    USERCMD = 5
    DATATABLES = 6
    STOP = 7
    CUSTOMDATA = 8
    STRINGTABLES = 9


cdef class DemoFile:

    cdef public unsigned int current_tick
    cdef list data_tables
    cdef list string_tables
    cdef list server_classes
    cdef public object game_events
    cdef object pending_baselines
    cdef public object instance_baselines
    cdef public object entities
    cdef object callbacks
    cdef dict internal_callbacks
    cdef dict merged_enums
    cdef dict user_messages
    cdef unsigned int server_class_bits
    cdef object byte_buf
    cdef object header

    def __cinit__(self, bytes data):
        self.header = DemoHeader.from_data(data[:1072])
        self.byte_buf = Bytebuffer(data[1072:])

        self.merged_enums = {
            v[1]: v[0] for v in netmessages_pb2.NET_Messages.items() +
            netmessages_pb2.SVC_Messages.items()
        }
        self.user_messages = {
            v[1]: v[0] for v in um.ECstrike15UserMessages.items()
        }
        self.current_tick = 0
        self.data_tables = []
        self.string_tables = []
        self.server_classes = []
        self.game_events = OrderedDict()
        self.pending_baselines = OrderedDict()
        self.instance_baselines = OrderedDict()
        self.entities = EntityList(self)
        self.callbacks = defaultdict(list)
        self.callbacks.update({
            'svc_GameEventList': [self.handle_game_event_list],
            'svc_GameEvent': [self.handle_game_event],
            'svc_CreateStringTable': [self.create_string_table],
            'svc_UpdateStringTable': [self.update_string_table],
            'svc_PacketEntities': [self.handle_packet_entities],
            'svc_UserMessage': [self.handle_user_message],
            'string_table_update': [self.table_updated]
        })

    cpdef void emit(self, str event, args=None):
        """Run callback functions for an event.

        Each time an event is trigged each registered callback
        will be called with the arguments provided to this method.
        """
        # cpdef functions don't support *args syntax
        if args is None:
            args = []

        for func in self.callbacks.get(event, []):
            func(*args)

    cpdef add_callback(self, event, func):
        """Register a callback function for an event."""
        self.callbacks[event].append(func)

    cpdef void parse(self) except *:
        """Parse the demofile.

        :emits: :ref:`tick_start <event_tick_start>`, \
                :ref:`tick_end <event_tick_end>`, \
                :ref:`end <event_end>`

        :raises: CommandError
        """
        while True:
            header = self.byte_buf.read_command_header()
            if header.tick != self.current_tick:
                self.emit('tick_end', [self.current_tick])
                self.current_tick = header.tick
                self.emit('tick_start', [self.current_tick])

            if header.command in (DemoCommand.SIGNON, DemoCommand.PACKET):
                self.handle_demo_packet(header)
            elif header.command == DemoCommand.SYNCTICK:
                continue
            elif header.command == DemoCommand.CONSOLECMD:
                length, buf = self.byte_buf.read_raw_data()
            elif header.command == DemoCommand.USERCMD:
                self.byte_buf.read_user_command()
            elif header.command == DemoCommand.DATATABLES:
                self.handle_data_table(header)
            elif header.command == DemoCommand.CUSTOMDATA:
                pass
            elif header.command == DemoCommand.STRINGTABLES:
                self.handle_string_tables(header)
            elif header.command == DemoCommand.STOP:
                self.emit('tick_end', [self.current_tick])
                self.emit('end')
                break
            else:
                raise CommandError("Unrecognized command")

    cpdef void handle_user_message(self, object msg):
        """Handle user message.

        A user message is stuff like text chat,
        voice chat, radio messages, etc.

        :emits: :ref:`user_message <event_user_msg>`
        """
        um_class = self.user_messages[msg.msg_type]
        class_name = 'CCSUsrMsg_{}'.format(um_class[6:])

        user_message = getattr(um, class_name)()
        user_message.ParseFromString(msg.msg_data)

        self.emit(um_class[6:], [user_message])

    cpdef void handle_packet_entities(self, object msg):
        """Create or update an entity.

        Entities represent in-game objects and are networked (i.e.
        their state is maintained on the server and updates are sent
        to one or more clients).

        Entities have a class ID which defines the type of entity
        it is. The class ID is also used to find the correct
        instance baseline. Instance baselines serve as default
        values when entities are created.
        """
        cdef unsigned int i = 0
        cdef int entity_idx = -1
        cdef Bitbuffer buf = Bitbuffer(msg.entity_data)

        for i in range(msg.updated_entries):
            entity_idx += 1 + buf.read_var_int()

            if buf.read_bit():
                if buf.read_bit():
                    self.entities[entity_idx] = None
            elif buf.read_bit():
                class_id = buf.read_uint_bits(self.server_class_bits)
                serial = buf.read_uint_bits(
                    consts.NUM_NETWORKED_EHANDLE_SERIAL_NUMBER_BITS
                )

                new_entity = self.entities.new_entity(
                    entity_idx, class_id, serial
                )
                self.read_new_entity(buf, new_entity)
            else:
                entity = self.entities[entity_idx]
                self.read_new_entity(buf, entity)

    cpdef void read_new_entity(self, Bitbuffer buf, object entity):
        """Read entity data.

        A list of updates is computed and for each one of those
        the appropritate property of the entity instance is updated.

        :emits: :ref:`change <event_change>`
        """
        cdef object server_class = self.server_classes[entity.class_id]
        cdef list updates = parse_entity_update(buf, server_class)

        for update in updates:
            table_name = update['prop']['table'].net_table_name
            var_name = update['prop']['prop'].var_name

            self.emit(
                'change', [entity, table_name, var_name, update['value']]
            )
            entity.update_prop(table_name, var_name, update['value'])

    cpdef void handle_demo_packet(self, cmd_header):
        """Handle demo packet.

        Demo packets are of type SVC\_ or NET\_. This method
        instantiates the appropriate protobuf class and triggers
        an event.

        :emits: :ref:`demo_packet <event_demo_packet>`.
        """
        self.byte_buf.read_command_data()
        self.byte_buf.read_sequence_data()

        for cmd, size, data in self.byte_buf.read_packet_data():
            cls = self.class_by_net_message_type(cmd)()

            cls.ParseFromString(data)
            self.emit(self.merged_enums[cmd], [cls])

    cpdef void handle_data_table(self, cmd_header):
        """Create data tables.

        Each entity class maintains a data table that describes
        how to encode each of its properties. These tables are
        called Send Tables and and genrally have names like
        DT_EntityClassName (DT_CSPlayer, DT_Weapon, etc.)

        :emits: :ref:`baseline_create <event_baseline_create>`, \
                :ref:`datatable_ready <event_datatable_ready>`.
        """
        cdef unsigned int i = 0
        # Size of entire data table chunk
        self.byte_buf.read(4)
        table = self.class_by_message_name('svc_SendTable')

        while True:
            # Type of table, not needed for now
            self.byte_buf.read_varint()

            data = self.byte_buf.read_var_bytes()

            msg = table()
            msg.ParseFromString(data)
            if msg.is_end:
                break

            self.data_tables.append(msg)

        server_classes = self.byte_buf.read_short()
        self.server_class_bits = <unsigned int>math.ceil(
            math.log2(server_classes)
        )

        for i in range(server_classes):
            class_id = self.byte_buf.read_short()
            assert class_id == i

            name = self.byte_buf.read_string()
            table_name = self.byte_buf.read_string()

            dt = self.data_table_by_name(table_name)

            table = {
                'class_id': class_id,
                'name': name,
                'table_name': table_name,
                'props': self.flatten_data_table(dt)
            }
            self.server_classes.append(table)

            # Handle pending baselines
            pending_baseline = self.pending_baselines.get(class_id)
            if pending_baseline:
                self.instance_baselines[class_id] = \
                    self.parse_instance_baseline(
                        pending_baseline, class_id
                )
                self.emit(
                    'baseline_created',
                    [class_id,
                     table,
                     self.instance_baselines[class_id]]
                )
                del self.pending_baselines[class_id]

        self.emit('datatable_ready', [table])

    cpdef void update_string_table(self, object msg):
        buf = Bitbuffer(msg.string_data)

        table = self.string_tables[msg.table_id]

        self.parse_string_table_update(
            buf, table, msg.num_changed_entries, len(table['entries']),
            0, False
        )

    cpdef void create_string_table(self, object msg):
        """Create a string table.

        A string table consists of a name and a list of entries.
        The maximum number of entries is created and initialized
        to None. This is done because future messages that update
        or reference string table entries do so by index.

        Each entry is a dictionary with two keys, entry and user_data.
        Entry is the name of the entry and user_data contains binary
        data related to the entry. In the 'user_info' string table
        user_data describes a UserInfo object.
        """
        cdef Bitbuffer buf = Bitbuffer(msg.string_data)

        table = {
            'name': msg.name,
            'entries': [{'entry': None, 'user_data': None}] * msg.max_entries
        }
        self.string_tables.append(table)
        self.parse_string_table_update(
            buf, table, msg.num_entries, msg.max_entries,
            msg.user_data_size_bits, msg.user_data_fixed_size
        )

    cpdef void parse_string_table_update(
            self, Bitbuffer buf, object table, unsigned int num_entries,
            unsigned int max_entries, unsigned int user_data_bits,
            bint user_data_fixed_size):
        """Update a string table.

        A single update message can update multiple table entries.
        Both the entry name and the user data can be updated.

        :emits: :ref:`string_table_update <event_string_table_update>`.
        """
        cdef unsigned int entry_bits = <unsigned int>math.log2(max_entries)
        cdef unsigned int i = 0
        history = deque(maxlen=32)

        dict_encode = buf.read_bit()
        assert not dict_encode, 'Dictionary encoding not supported'

        for i in range(num_entries):
            index = i
            entry = None

            if not buf.read_bit():
                index = buf.read_uint_bits(entry_bits)

            assert index >=0 and index <= max_entries

            # entry changed?
            changed = buf.read_bit()
            if changed:
                # substring check
                substr = buf.read_bit()
                if substr:
                    idx = buf.read_uint_bits(5)
                    bytes_to_copy = buf.read_uint_bits(consts.SUBSTRING_BITS)
                    substring = history[idx][:bytes_to_copy * 8]
                    suffix = buf.read_string()
                    entry = substring + suffix
                else:
                    entry = buf.read_string()

                table['entries'][index]['entry'] = entry

            # Deal with user data
            user_data = None
            if buf.read_bit():
                if user_data_fixed_size:
                    user_data = buf.read_user_data(user_data_bits)
                else:
                    size = buf.read_uint_bits(consts.MAX_USERDATA_BITS)
                    user_data = buf.read_user_data(size * 8)

                if table['name'] == 'userinfo':
                    user_data = UserInfo.from_data(user_data)

                table['entries'][index]['user_data'] = user_data

            history.append(entry)
            self.emit(
                'string_table_update', [table, index, entry, user_data]
            )

    cpdef void handle_string_tables(self, object cmd_header):
        cdef Bitbuffer buf = self.byte_buf.read_bitstream()
        cdef unsigned char num_tables = buf.read_uint_bits(8)
        cdef unsigned char i = 0

        for i in range(num_tables):
            table_name = buf.read_string()
            self.handle_string_table(table_name, buf)

    cpdef void handle_string_table(self, str table_name, Bitbuffer buf):
        """Populate a string table.

        :emits: :ref:`string_table_update <event_string_table_update>`.
        """
        cdef unsigned short entries = buf.read_uint_bits(16)
        cdef unsigned short entry_idx = 0
        table = self.table_by_name(table_name)

        for entry_idx in range(entries):
            entry_name = buf.read_string()

            one_bit = buf.read_bit()
            user_data = None
            if one_bit:
                user_data_len = buf.read_uint_bits(16)
                user_data = buf.read_user_data(user_data_len * 8)
                if table_name == 'userinfo':
                    user_data = UserInfo.from_data(user_data)

            table['entries'][entry_idx] = {
                'entry': entry_name,
                'user_data': user_data
            }

            self.emit(
                'string_table_update',
                [table, entry_idx, entry_name, user_data]
            )

        # Client-side entries, maybe they don't exist? I've never seen them
        if buf.read_bit():
            num_strings = buf.read_uint_bits(16)

            for string in range(num_strings):
                # entry name
                buf.read_string()
                user_data = None

                if buf.read_bit():
                    user_data_len = buf.read_uint_bits(16)
                    user_data = buf.read_user_data(user_data_len * 8)

    cpdef void handle_game_event_list(self, object msg):
        """Create game event list.

        This message contains a list of game events present in
        this replay. Later, when a game event occurs its ID can
        be used to get a game event object.
        """
        for event in msg.descriptors:
            self.game_events.update({
                event.eventid: {
                    'name': event.name,
                    'event': event
                }
            })

    cpdef void handle_game_event(self, object msg):
        """Handle game event.

        A game event message just contains an event ID.
        The corresponding event can be found in self.game_events.
        An event is then triggered for the game event itself.

        :emits: :ref:`game_event <event_game_event>`.
        """
        event = self.game_events[msg.eventid]
        self.emit(event['name'], [event, msg])

    cdef object class_by_net_message_type(self, unsigned int msg_type):
        """Find a Protobuf class for the given message type.

        The name for the message type is found and then converted
        to the Protobuf class naming convention.

        :returns: Protobuf class
        :raises: ClassNotFoundError
        """
        cls = self.merged_enums[msg_type]

        if not cls:
            raise ClassNotFoundError(
                "Class for message type {} not found.".format(msg_type)
            )

        enum_type = cls[:3]
        enum_name = cls[4:]
        class_name = 'C{}Msg_{}'.format(enum_type.upper(), enum_name)

        return getattr(netmessages_pb2, class_name)

    cpdef object class_by_message_name(self, str name):
        """Find a Protobuf class for the given message.

        The generated Protobuf modules create classes like
        C<type>Msg_<name>. For a given message name this
        method will convert the name to a Protobuf class name
        and return it.

        :returns: Protobuf class
        """
        enum_type = name[:3]
        enum_name = name[4:]
        class_name = 'C{}Msg_{}'.format(enum_type.upper(), enum_name)
        return getattr(netmessages_pb2, class_name)

    cpdef object table_by_name(self, str name):
        return [t for t in self.string_tables if t['name'] == name][0]

    cpdef object data_table_by_name(self, str name):
        return [t for t in self.data_tables if t.net_table_name == name][0]

    cpdef void table_updated(self, table, index, entry, user_data):
        """Update an instance baseline.

        Each time a string table is updated this method is
        called. If a baseline doesn't exist yet it is added to
        the list of pending baselines which are handled later
        when the correct server class is created. Server classes
        are created by DATATABLES commands.
        """
        if table['name'] != 'instancebaseline' or not user_data:
            return

        cdef unsigned int class_id = int(entry)
        cdef Bitbuffer baseline_buf = Bitbuffer(user_data)

        try:
            self.server_classes[class_id]
        except IndexError:
            self.pending_baselines[class_id] = baseline_buf
            return

        self.instance_baselines[class_id] = self.parse_instance_baseline(
            baseline_buf, class_id
        )

    cpdef object parse_instance_baseline(self, Bitbuffer buf,
                                         unsigned int class_id):
        """Parse baseline.

        Baselines serve as default data for entities of type
        `class_id`.

        :param buf: Data to read
        :type buf: Bitbuffer
        :param class_id: Server class ID
        :type class_id: unsigned int

        :returns: baseline for `class_id`.
        """
        class_baseline = OrderedDict()
        server_class = self.server_classes[class_id]

        for baseline in parse_entity_update(buf, server_class):
            table_name = baseline['prop']['table'].net_table_name
            var_name = baseline['prop']['prop'].var_name

            if table_name not in class_baseline:
                class_baseline[table_name] = OrderedDict()
            class_baseline[table_name][var_name] = baseline['value']

        return class_baseline

    cpdef object flatten_data_table(self, table):
        """Flatten a data table.

        Data tables can contain other data tables as entries.
        This method collects all properties and sorts them
        by priority.

        :returns: Flattened data table
        """
        flattened_props = self.collect_props(
            table, self.collect_exclusions(table)
        )

        priorities = set([p['prop'].priority for p in flattened_props])
        priorities.add(64)
        priorities = sorted(list(priorities))

        start = 0
        for prio in priorities:
            while True:
                current_prop = start
                while current_prop < len(flattened_props):
                    prop = flattened_props[current_prop]['prop']
                    if (prop.priority == prio or (prio == 64 and
                       (prop.flags & PropFlags.SPROP_CHANGES_OFTEN))):
                        if start != current_prop:
                            temp = flattened_props[start]
                            flattened_props[start] = \
                                flattened_props[current_prop]
                            flattened_props[current_prop] = temp
                        start += 1
                        break
                    current_prop += 1
                if current_prop == len(flattened_props):
                    break

        # Doesn't seem to work
        # props = sorted(flattened_props, key=lambda x: x['prop'].priority)
        return flattened_props

    cdef bint _is_prop_excluded(self, list exclusions,
                                object table, object prop):
        for exclusion in exclusions:
            if (table.net_table_name == exclusion.dt_name and
               prop.var_name == exclusion.var_name):
                return True

    cpdef list collect_exclusions(self, object table):
        exclusions = []
        cdef unsigned int idx

        for idx, prop in enumerate(table.props):
            if prop.flags & PropFlags.SPROP_EXCLUDE:
                exclusions.append(prop)

            if prop.type == PropTypes.DPT_DataTable:
                sub_table = self.data_table_by_name(prop.dt_name)
                exclusions.extend(self.collect_exclusions(sub_table))
        return exclusions

    cpdef list collect_props(self, object table, list exclusions):
        """Collect table properties which are not excluded.

        If a property is part of an array or is flagged as
        excluded it will not be returned. Otherwise all
        properties are collected and data table entries are
        recursively collected.

        :returns: Flat list of all table properties
        """
        flattened = []
        cdef unsigned int idx

        for idx, prop in enumerate(table.props):

            if (prop.flags & PropFlags.SPROP_INSIDEARRAY or
               prop.flags & PropFlags.SPROP_EXCLUDE or
               self._is_prop_excluded(exclusions, table, prop)):
                continue

            if prop.type == PropTypes.DPT_DataTable:
                sub_table = self.data_table_by_name(prop.dt_name)
                child_props = self.collect_props(sub_table, exclusions)

                if prop.flags & PropFlags.SPROP_COLLAPSIBLE == 0:
                    for cp in child_props:
                        cp['collapsible'] = False

                flattened.extend(child_props)
            elif prop.type == PropTypes.DPT_Array:
                flattened.append({
                    'prop': prop,
                    'array_element_prop': table.props[idx - 1],
                    'table': table
                })
            else:
                flattened.append({
                    'prop': prop,
                    'table': table
                })

        return sorted(flattened, key=_key_sort)



import pytest

from demoparser.demofile import CommandError
from demoparser.demofile import DemoFile
from demoparser.structures import CommandHeader


def test_parse_invalid_command():
    # Demo files have a 1072 byte header that isn't needed here
    data = b'HL2DEMO\x00' + bytes([0] * 1064)
    header = CommandHeader()
    header.player = 1
    header.tick = 1
    header.command = 99

    df = DemoFile(data + header.pack())
    with pytest.raises(CommandError) as exc:
        df.parse()

    assert 'Unrecognized command' in str(exc.value)



class BaseEntity:

    def __init__(self, parser, index, class_id, serial, props):
        self.parser = parser
        self.index = index
        self.class_id = class_id
        self.serial = serial
        self.props = props

    def update_prop(self, table, key, value):
        """Update entity property."""
        if table in self.props and key in self.props[table]:
            self.props[table][key] = value

    def get_prop(self, table, var):
        return self.props.get(table, {}).get(var)

    @property
    def position(self):
        """Get the position of this entity."""
        return self.get_prop('DT_BaseEntity', 'm_vecOrigin')

    @property
    def server_class(self):
        """Server class to which this entity belongs."""
        return self.parser.server_classes[self.class_id]

    @property
    def team_num(self):
        """Entity team number.

        :returns: Team number of entity.
        """
        return self.get_prop('DT_BaseEntity', 'm_iTeamNum')

    @property
    def team(self):
        """Team to which entity belongs.

        :returns: Team entity
        """
        team_num = self.team_num

        if not team_num:
            return

        return self.parser.entities.teams[team_num]

    @property
    def owner(self):
        """Find entity which owns this entity.

        For example, a weapon entity belongs to a player entity.
        Calling this method on a weapon entity would return a
        player.

        :returns: Owning entity.
        """
        return self.parser.entities.get_by_handle(
            self.get_prop('DT_BaseEntity', 'm_hOwnerEntity')
        )

    def properties(self):
        result = {
            'index': self.index,
            'class_id': self.class_id,
            'serial': self.serial,
            'position': self.position,
        }

        return result



from demoparser.entities import BaseEntity
import inspect


class Player(BaseEntity):

    def __init__(self, parser, index, class_id, serial, props):
        self.parser = parser
        self.index = index
        self.slot = self.index - 1
        self.class_id = class_id
        self.serial = serial
        self.props = props
        self.user_info = self._get_user_info()
        self._serialize_props = [

        ]

    def _get_user_info(self):
        users = self.parser.table_by_name('userinfo')['entries']
        return users[self.slot]['user_data']

    @property
    def health(self):
        """Get current health."""
        return self.get_prop('DT_BasePlayer', 'm_iHealth')

    @property
    def name(self):
        """Get player's name."""
        if isinstance(self.user_info, bytes):
            return ""
        return self.user_info.name.decode('utf-8')

    @property
    def steam_id(self):
        """Get Steam ID."""
        return self.user_info.guid.decode('utf-8')

    @property
    def position(self):
        """Get current position.

        :returns: Position vector.
        """
        vec = self.get_prop('DT_CSLocalPlayerExclusive', 'm_vecOrigin')
        z = self.get_prop('DT_CSLocalPlayerExclusive', 'm_vecOrigin[2]')

        return {
            'x': vec['x'],
            'y': vec['y'],
            'z': z
        }

    @property
    def view_angle(self):
        """Get current view angle.

        :returns: Tuple of pitch and yaw.
        """
        pitch = self.get_prop('DT_CSPlayer', 'm_angEyeAngles[0]')
        yaw = self.get_prop('DT_CSPlayer', 'm_angEyeAngles[1]')

        return {'pitch': pitch, 'yaw': yaw}

    @property
    def cash(self):
        """Get """
        return self.get_prop('DT_CSPlayer', 'm_iAccount')

    @property
    def life_state(self):
        """Get life state.

        0
          Alive.

        1
          Dying. Either the death animation is still playing
          or the player is falling and waiting to hit the ground.

        2
          Dead. Not moving.

        :returns: Life state.
        """
        return self.get_prop('DT_BasePlayer', 'm_lifeState')

    @property
    def armor(self):
        """Get armor value."""
        return self.get_prop('DT_CSPlayer', 'm_ArmorValue')

    @property
    def place(self):
        """Get last place player occupied.

        A place refers a named navigation mesh. A navigation mesh
        represents the walkable areas of a map.

        :returns: Name of last nav mesh occupied.
        """
        return self.get_prop('DT_BasePlayer', 'm_szLastPlaceName')

    @property
    def kills(self):
        """Number of kills for this player."""
        if isinstance(self.user_info, bytes):
            return 0
        prop = self.parser.entities.get_one('DT_CSPlayerResource')
        # This property has a list of kills for all players
        kills = prop.props['m_iKills']
        player_id = str(self.user_info.user_id).zfill(3)
        return kills[player_id]

    @property
    def weapon(self):
        """Get current weapon."""
        return self.parser.entities.get_by_handle(
            self.get_prop('DT_BaseCombatCharacter', 'm_hActiveWeapon')
        )

    def properties(self):
        result = {
            'index': self.index,
            'class_id': self.class_id,
            'serial': self.serial,
        }

        for name, value in inspect.getmembers(self):
            prop_attr = getattr(self.__class__, name, None)
            if inspect.isdatadescriptor(prop_attr):
                attr = getattr(self, name, None)
                if not isinstance(attr, BaseEntity):
                    result[name] = value

        return result



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: demoparser/protobufs/network_connection.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='demoparser/protobufs/network_connection.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n-demoparser/protobufs/network_connection.proto\x1a google/protobuf/descriptor.proto*\xd7%\n\x1b\x45NetworkDisconnectionReason\x12\x1e\n\x1aNETWORK_DISCONNECT_INVALID\x10\x00\x12\x1f\n\x1bNETWORK_DISCONNECT_SHUTDOWN\x10\x01\x12\x46\n%NETWORK_DISCONNECT_DISCONNECT_BY_USER\x10\x02\x1a\x1b\xa2\xd4\x18\x17#GameUI_Disconnect_User\x12J\n\'NETWORK_DISCONNECT_DISCONNECT_BY_SERVER\x10\x03\x1a\x1d\xa2\xd4\x18\x19#GameUI_Disconnect_Server\x12\x42\n\x17NETWORK_DISCONNECT_LOST\x10\x04\x1a%\xa2\xd4\x18!#GameUI_Disconnect_ConnectionLost\x12J\n\x1bNETWORK_DISCONNECT_OVERFLOW\x10\x05\x1a)\xa2\xd4\x18%#GameUI_Disconnect_ConnectionOverflow\x12I\n\x1fNETWORK_DISCONNECT_STEAM_BANNED\x10\x06\x1a$\xa2\xd4\x18 #GameUI_Disconnect_SteamIDBanned\x12G\n\x1eNETWORK_DISCONNECT_STEAM_INUSE\x10\x07\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_SteamIDInUse\x12G\n\x1fNETWORK_DISCONNECT_STEAM_TICKET\x10\x08\x1a\"\xa2\xd4\x18\x1e#GameUI_Disconnect_SteamTicket\x12\x45\n\x1eNETWORK_DISCONNECT_STEAM_LOGON\x10\t\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_SteamLogon\x12M\n&NETWORK_DISCONNECT_STEAM_AUTHCANCELLED\x10\n\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_SteamLogon\x12O\n(NETWORK_DISCONNECT_STEAM_AUTHALREADYUSED\x10\x0b\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_SteamLogon\x12K\n$NETWORK_DISCONNECT_STEAM_AUTHINVALID\x10\x0c\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_SteamLogon\x12I\n$NETWORK_DISCONNECT_STEAM_VACBANSTATE\x10\r\x1a\x1f\xa2\xd4\x18\x1b#GameUI_Disconnect_SteamVAC\x12S\n,NETWORK_DISCONNECT_STEAM_LOGGED_IN_ELSEWHERE\x10\x0e\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_SteamInUse\x12T\n+NETWORK_DISCONNECT_STEAM_VAC_CHECK_TIMEDOUT\x10\x0f\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_SteamTimeOut\x12I\n NETWORK_DISCONNECT_STEAM_DROPPED\x10\x10\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_SteamDropped\x12M\n\"NETWORK_DISCONNECT_STEAM_OWNERSHIP\x10\x11\x1a%\xa2\xd4\x18!#GameUI_Disconnect_SteamOwnership\x12U\n&NETWORK_DISCONNECT_SERVERINFO_OVERFLOW\x10\x12\x1a)\xa2\xd4\x18%#GameUI_Disconnect_ServerInfoOverflow\x12K\n#NETWORK_DISCONNECT_TICKMSG_OVERFLOW\x10\x13\x1a\"\xa2\xd4\x18\x1e#GameUI_Disconnect_TickMessage\x12Y\n*NETWORK_DISCONNECT_STRINGTABLEMSG_OVERFLOW\x10\x14\x1a)\xa2\xd4\x18%#GameUI_Disconnect_StringTableMessage\x12S\n\'NETWORK_DISCONNECT_DELTAENTMSG_OVERFLOW\x10\x15\x1a&\xa2\xd4\x18\"#GameUI_Disconnect_DeltaEntMessage\x12Q\n&NETWORK_DISCONNECT_TEMPENTMSG_OVERFLOW\x10\x16\x1a%\xa2\xd4\x18!#GameUI_Disconnect_TempEntMessage\x12O\n%NETWORK_DISCONNECT_SOUNDSMSG_OVERFLOW\x10\x17\x1a$\xa2\xd4\x18 #GameUI_Disconnect_SoundsMessage\x12P\n#NETWORK_DISCONNECT_SNAPSHOTOVERFLOW\x10\x18\x1a\'\xa2\xd4\x18##GameUI_Disconnect_SnapshotOverflow\x12J\n NETWORK_DISCONNECT_SNAPSHOTERROR\x10\x19\x1a$\xa2\xd4\x18 #GameUI_Disconnect_SnapshotError\x12P\n#NETWORK_DISCONNECT_RELIABLEOVERFLOW\x10\x1a\x1a\'\xa2\xd4\x18##GameUI_Disconnect_ReliableOverflow\x12N\n\x1fNETWORK_DISCONNECT_BADDELTATICK\x10\x1b\x1a)\xa2\xd4\x18%#GameUI_Disconnect_BadClientDeltaTick\x12H\n\x1fNETWORK_DISCONNECT_NOMORESPLITS\x10\x1c\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_NoMoreSplits\x12@\n\x1bNETWORK_DISCONNECT_TIMEDOUT\x10\x1d\x1a\x1f\xa2\xd4\x18\x1b#GameUI_Disconnect_TimedOut\x12H\n\x1fNETWORK_DISCONNECT_DISCONNECTED\x10\x1e\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_Disconnected\x12H\n\x1fNETWORK_DISCONNECT_LEAVINGSPLIT\x10\x1f\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_LeavingSplit\x12X\n\'NETWORK_DISCONNECT_DIFFERENTCLASSTABLES\x10 \x1a+\xa2\xd4\x18\'#GameUI_Disconnect_DifferentClassTables\x12P\n#NETWORK_DISCONNECT_BADRELAYPASSWORD\x10!\x1a\'\xa2\xd4\x18##GameUI_Disconnect_BadRelayPassword\x12X\n\'NETWORK_DISCONNECT_BADSPECTATORPASSWORD\x10\"\x1a+\xa2\xd4\x18\'#GameUI_Disconnect_BadSpectatorPassword\x12L\n!NETWORK_DISCONNECT_HLTVRESTRICTED\x10#\x1a%\xa2\xd4\x18!#GameUI_Disconnect_HLTVRestricted\x12H\n\x1fNETWORK_DISCONNECT_NOSPECTATORS\x10$\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_NoSpectators\x12N\n\"NETWORK_DISCONNECT_HLTVUNAVAILABLE\x10%\x1a&\xa2\xd4\x18\"#GameUI_Disconnect_HLTVUnavailable\x12@\n\x1bNETWORK_DISCONNECT_HLTVSTOP\x10&\x1a\x1f\xa2\xd4\x18\x1b#GameUI_Disconnect_HLTVStop\x12<\n\x19NETWORK_DISCONNECT_KICKED\x10\'\x1a\x1d\xa2\xd4\x18\x19#GameUI_Disconnect_Kicked\x12@\n\x1bNETWORK_DISCONNECT_BANADDED\x10(\x1a\x1f\xa2\xd4\x18\x1b#GameUI_Disconnect_BanAdded\x12H\n\x1fNETWORK_DISCONNECT_KICKBANADDED\x10)\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_KickBanAdded\x12\x44\n\x1dNETWORK_DISCONNECT_HLTVDIRECT\x10*\x1a!\xa2\xd4\x18\x1d#GameUI_Disconnect_HLTVDirect\x12\\\n)NETWORK_DISCONNECT_PURESERVER_CLIENTEXTRA\x10+\x1a-\xa2\xd4\x18)#GameUI_Disconnect_PureServer_ClientExtra\x12V\n&NETWORK_DISCONNECT_PURESERVER_MISMATCH\x10,\x1a*\xa2\xd4\x18&#GameUI_Disconnect_PureServer_Mismatch\x12>\n\x1aNETWORK_DISCONNECT_USERCMD\x10-\x1a\x1e\xa2\xd4\x18\x1a#GameUI_Disconnect_UserCmd\x12N\n#NETWORK_DISCONNECT_REJECTED_BY_GAME\x10.\x1a%\xa2\xd4\x18!#GameUI_Disconnect_RejectedByGame\x12T\n&NETWORK_DISCONNECT_MESSAGE_PARSE_ERROR\x10/\x1a(\xa2\xd4\x18$#GameUI_Disconnect_MessageParseError\x12X\n(NETWORK_DISCONNECT_INVALID_MESSAGE_ERROR\x10\x30\x1a*\xa2\xd4\x18&#GameUI_Disconnect_InvalidMessageError\x12T\n&NETWORK_DISCONNECT_BAD_SERVER_PASSWORD\x10\x31\x1a(\xa2\xd4\x18$#GameUI_Disconnect_BadServerPassword\x12\x62\n-NETWORK_DISCONNECT_DIRECT_CONNECT_RESERVATION\x10\x32\x1a/\xa2\xd4\x18+#GameUI_Disconnect_DirectConnectReservation\x12S\n%NETWORK_DISCONNECT_CONNECTION_FAILURE\x10\x33\x1a(\xa2\xd4\x18$#GameUI_Disconnect_ConnectionFailure\x12Y\n)NETWORK_DISCONNECT_NO_PEER_GROUP_HANDLERS\x10\x34\x1a*\xa2\xd4\x18&#GameUI_Disconnect_NoPeerGroupHandlers\x12H\n\x1fNETWORK_DISCONNECT_RECONNECTION\x10\x35\x1a#\xa2\xd4\x18\x1f#GameUI_Disconnect_Reconnection\x12S\n%NETWORK_DISCONNECT_CONNECTION_CLOSING\x10\x36\x1a(\xa2\xd4\x18$#GameUI_Disconnect_ConnectionClosing\x12]\n+NETWORK_DISCONNECT_NO_GOTV_RELAYS_AVAILABLE\x10\x37\x1a,\xa2\xd4\x18(#GameUI_Disconnect_NoGOTVRelaysAvailable\x12O\n#NETWORK_DISCONNECT_SESSION_MIGRATED\x10\x38\x1a&\xa2\xd4\x18\"#GameUI_Disconnect_SessionMigrated\x12\x62\n,NETWORK_DISCONNECT_VERYLARGETRANSFEROVERFLOW\x10\x39\x1a\x30\xa2\xd4\x18,#GameUI_Disconnect_VeryLargeTransferOverflow\x12N\n\"NETWORK_DISCONNECT_SENDNETOVERFLOW\x10:\x1a&\xa2\xd4\x18\"#GameUI_Disconnect_SendNetOverflow\x12l\n3NETWORK_DISCONNECT_PLAYER_REMOVED_FROM_HOST_SESSION\x10;\x1a\x33\xa2\xd4\x18/#GameUI_Disconnect_PlayerRemovedFromHostSession\x12I\n\x1dNETWORK_DISCONNECT_SERVER_DOS\x10=\x1a&\xa2\xd4\x18\"#GameUI_Disconnect_TooManyCommands:E\n\x18network_connection_token\x12!.google.protobuf.EnumValueOptions\x18\xc4\x8a\x03 \x01(\tB\x03\x80\x01\x00')
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_ENETWORKDISCONNECTIONREASON = _descriptor.EnumDescriptor(
  name='ENetworkDisconnectionReason',
  full_name='ENetworkDisconnectionReason',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_INVALID', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SHUTDOWN', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DISCONNECT_BY_USER', index=2, number=2,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\027#GameUI_Disconnect_User')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DISCONNECT_BY_SERVER', index=3, number=3,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\031#GameUI_Disconnect_Server')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_LOST', index=4, number=4,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_ConnectionLost')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_OVERFLOW', index=5, number=5,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_ConnectionOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_BANNED', index=6, number=6,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SteamIDBanned')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_INUSE', index=7, number=7,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamIDInUse')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_TICKET', index=8, number=8,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\036#GameUI_Disconnect_SteamTicket')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_LOGON', index=9, number=9,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_AUTHCANCELLED', index=10, number=10,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_AUTHALREADYUSED', index=11, number=11,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_AUTHINVALID', index=12, number=12,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_VACBANSTATE', index=13, number=13,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_SteamVAC')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_LOGGED_IN_ELSEWHERE', index=14, number=14,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamInUse')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_VAC_CHECK_TIMEDOUT', index=15, number=15,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamTimeOut')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_DROPPED', index=16, number=16,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamDropped')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STEAM_OWNERSHIP', index=17, number=17,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_SteamOwnership')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SERVERINFO_OVERFLOW', index=18, number=18,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_ServerInfoOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_TICKMSG_OVERFLOW', index=19, number=19,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\036#GameUI_Disconnect_TickMessage')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_STRINGTABLEMSG_OVERFLOW', index=20, number=20,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_StringTableMessage')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DELTAENTMSG_OVERFLOW', index=21, number=21,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_DeltaEntMessage')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_TEMPENTMSG_OVERFLOW', index=22, number=22,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_TempEntMessage')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SOUNDSMSG_OVERFLOW', index=23, number=23,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SoundsMessage')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SNAPSHOTOVERFLOW', index=24, number=24,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_SnapshotOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SNAPSHOTERROR', index=25, number=25,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SnapshotError')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_RELIABLEOVERFLOW', index=26, number=26,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_ReliableOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_BADDELTATICK', index=27, number=27,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_BadClientDeltaTick')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_NOMORESPLITS', index=28, number=28,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_NoMoreSplits')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_TIMEDOUT', index=29, number=29,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_TimedOut')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DISCONNECTED', index=30, number=30,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_Disconnected')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_LEAVINGSPLIT', index=31, number=31,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_LeavingSplit')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DIFFERENTCLASSTABLES', index=32, number=32,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\'#GameUI_Disconnect_DifferentClassTables')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_BADRELAYPASSWORD', index=33, number=33,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_BadRelayPassword')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_BADSPECTATORPASSWORD', index=34, number=34,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\'#GameUI_Disconnect_BadSpectatorPassword')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_HLTVRESTRICTED', index=35, number=35,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_HLTVRestricted')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_NOSPECTATORS', index=36, number=36,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_NoSpectators')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_HLTVUNAVAILABLE', index=37, number=37,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_HLTVUnavailable')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_HLTVSTOP', index=38, number=38,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_HLTVStop')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_KICKED', index=39, number=39,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\031#GameUI_Disconnect_Kicked')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_BANADDED', index=40, number=40,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_BanAdded')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_KICKBANADDED', index=41, number=41,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_KickBanAdded')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_HLTVDIRECT', index=42, number=42,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_HLTVDirect')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_PURESERVER_CLIENTEXTRA', index=43, number=43,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030)#GameUI_Disconnect_PureServer_ClientExtra')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_PURESERVER_MISMATCH', index=44, number=44,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_PureServer_Mismatch')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_USERCMD', index=45, number=45,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\032#GameUI_Disconnect_UserCmd')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_REJECTED_BY_GAME', index=46, number=46,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_RejectedByGame')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_MESSAGE_PARSE_ERROR', index=47, number=47,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_MessageParseError')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_INVALID_MESSAGE_ERROR', index=48, number=48,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_InvalidMessageError')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_BAD_SERVER_PASSWORD', index=49, number=49,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_BadServerPassword')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_DIRECT_CONNECT_RESERVATION', index=50, number=50,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030+#GameUI_Disconnect_DirectConnectReservation')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_CONNECTION_FAILURE', index=51, number=51,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_ConnectionFailure')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_NO_PEER_GROUP_HANDLERS', index=52, number=52,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_NoPeerGroupHandlers')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_RECONNECTION', index=53, number=53,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_Reconnection')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_CONNECTION_CLOSING', index=54, number=54,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_ConnectionClosing')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_NO_GOTV_RELAYS_AVAILABLE', index=55, number=55,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030(#GameUI_Disconnect_NoGOTVRelaysAvailable')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SESSION_MIGRATED', index=56, number=56,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_SessionMigrated')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_VERYLARGETRANSFEROVERFLOW', index=57, number=57,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030,#GameUI_Disconnect_VeryLargeTransferOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SENDNETOVERFLOW', index=58, number=58,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_SendNetOverflow')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_PLAYER_REMOVED_FROM_HOST_SESSION', index=59, number=59,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030/#GameUI_Disconnect_PlayerRemovedFromHostSession')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NETWORK_DISCONNECT_SERVER_DOS', index=60, number=61,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_TooManyCommands')),
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=84,
  serialized_end=4907,
)
_sym_db.RegisterEnumDescriptor(_ENETWORKDISCONNECTIONREASON)

ENetworkDisconnectionReason = enum_type_wrapper.EnumTypeWrapper(_ENETWORKDISCONNECTIONREASON)
NETWORK_DISCONNECT_INVALID = 0
NETWORK_DISCONNECT_SHUTDOWN = 1
NETWORK_DISCONNECT_DISCONNECT_BY_USER = 2
NETWORK_DISCONNECT_DISCONNECT_BY_SERVER = 3
NETWORK_DISCONNECT_LOST = 4
NETWORK_DISCONNECT_OVERFLOW = 5
NETWORK_DISCONNECT_STEAM_BANNED = 6
NETWORK_DISCONNECT_STEAM_INUSE = 7
NETWORK_DISCONNECT_STEAM_TICKET = 8
NETWORK_DISCONNECT_STEAM_LOGON = 9
NETWORK_DISCONNECT_STEAM_AUTHCANCELLED = 10
NETWORK_DISCONNECT_STEAM_AUTHALREADYUSED = 11
NETWORK_DISCONNECT_STEAM_AUTHINVALID = 12
NETWORK_DISCONNECT_STEAM_VACBANSTATE = 13
NETWORK_DISCONNECT_STEAM_LOGGED_IN_ELSEWHERE = 14
NETWORK_DISCONNECT_STEAM_VAC_CHECK_TIMEDOUT = 15
NETWORK_DISCONNECT_STEAM_DROPPED = 16
NETWORK_DISCONNECT_STEAM_OWNERSHIP = 17
NETWORK_DISCONNECT_SERVERINFO_OVERFLOW = 18
NETWORK_DISCONNECT_TICKMSG_OVERFLOW = 19
NETWORK_DISCONNECT_STRINGTABLEMSG_OVERFLOW = 20
NETWORK_DISCONNECT_DELTAENTMSG_OVERFLOW = 21
NETWORK_DISCONNECT_TEMPENTMSG_OVERFLOW = 22
NETWORK_DISCONNECT_SOUNDSMSG_OVERFLOW = 23
NETWORK_DISCONNECT_SNAPSHOTOVERFLOW = 24
NETWORK_DISCONNECT_SNAPSHOTERROR = 25
NETWORK_DISCONNECT_RELIABLEOVERFLOW = 26
NETWORK_DISCONNECT_BADDELTATICK = 27
NETWORK_DISCONNECT_NOMORESPLITS = 28
NETWORK_DISCONNECT_TIMEDOUT = 29
NETWORK_DISCONNECT_DISCONNECTED = 30
NETWORK_DISCONNECT_LEAVINGSPLIT = 31
NETWORK_DISCONNECT_DIFFERENTCLASSTABLES = 32
NETWORK_DISCONNECT_BADRELAYPASSWORD = 33
NETWORK_DISCONNECT_BADSPECTATORPASSWORD = 34
NETWORK_DISCONNECT_HLTVRESTRICTED = 35
NETWORK_DISCONNECT_NOSPECTATORS = 36
NETWORK_DISCONNECT_HLTVUNAVAILABLE = 37
NETWORK_DISCONNECT_HLTVSTOP = 38
NETWORK_DISCONNECT_KICKED = 39
NETWORK_DISCONNECT_BANADDED = 40
NETWORK_DISCONNECT_KICKBANADDED = 41
NETWORK_DISCONNECT_HLTVDIRECT = 42
NETWORK_DISCONNECT_PURESERVER_CLIENTEXTRA = 43
NETWORK_DISCONNECT_PURESERVER_MISMATCH = 44
NETWORK_DISCONNECT_USERCMD = 45
NETWORK_DISCONNECT_REJECTED_BY_GAME = 46
NETWORK_DISCONNECT_MESSAGE_PARSE_ERROR = 47
NETWORK_DISCONNECT_INVALID_MESSAGE_ERROR = 48
NETWORK_DISCONNECT_BAD_SERVER_PASSWORD = 49
NETWORK_DISCONNECT_DIRECT_CONNECT_RESERVATION = 50
NETWORK_DISCONNECT_CONNECTION_FAILURE = 51
NETWORK_DISCONNECT_NO_PEER_GROUP_HANDLERS = 52
NETWORK_DISCONNECT_RECONNECTION = 53
NETWORK_DISCONNECT_CONNECTION_CLOSING = 54
NETWORK_DISCONNECT_NO_GOTV_RELAYS_AVAILABLE = 55
NETWORK_DISCONNECT_SESSION_MIGRATED = 56
NETWORK_DISCONNECT_VERYLARGETRANSFEROVERFLOW = 57
NETWORK_DISCONNECT_SENDNETOVERFLOW = 58
NETWORK_DISCONNECT_PLAYER_REMOVED_FROM_HOST_SESSION = 59
NETWORK_DISCONNECT_SERVER_DOS = 61

NETWORK_CONNECTION_TOKEN_FIELD_NUMBER = 50500
network_connection_token = _descriptor.FieldDescriptor(
  name='network_connection_token', full_name='network_connection_token', index=0,
  number=50500, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=_b("").decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None)

DESCRIPTOR.enum_types_by_name['ENetworkDisconnectionReason'] = _ENETWORKDISCONNECTIONREASON
DESCRIPTOR.extensions_by_name['network_connection_token'] = network_connection_token

google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(network_connection_token)

DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\200\001\000'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECT_BY_USER"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECT_BY_USER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\027#GameUI_Disconnect_User'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECT_BY_SERVER"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECT_BY_SERVER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\031#GameUI_Disconnect_Server'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_LOST"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_LOST"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_ConnectionLost'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_ConnectionOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_BANNED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_BANNED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SteamIDBanned'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_INUSE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_INUSE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamIDInUse'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_TICKET"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_TICKET"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\036#GameUI_Disconnect_SteamTicket'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_LOGON"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_LOGON"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHCANCELLED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHCANCELLED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHALREADYUSED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHALREADYUSED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHINVALID"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_AUTHINVALID"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamLogon'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_VACBANSTATE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_VACBANSTATE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_SteamVAC'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_LOGGED_IN_ELSEWHERE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_LOGGED_IN_ELSEWHERE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_SteamInUse'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_VAC_CHECK_TIMEDOUT"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_VAC_CHECK_TIMEDOUT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamTimeOut'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_DROPPED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_DROPPED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_SteamDropped'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_OWNERSHIP"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STEAM_OWNERSHIP"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_SteamOwnership'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SERVERINFO_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SERVERINFO_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_ServerInfoOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TICKMSG_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TICKMSG_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\036#GameUI_Disconnect_TickMessage'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STRINGTABLEMSG_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_STRINGTABLEMSG_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_StringTableMessage'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DELTAENTMSG_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DELTAENTMSG_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_DeltaEntMessage'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TEMPENTMSG_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TEMPENTMSG_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_TempEntMessage'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SOUNDSMSG_OVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SOUNDSMSG_OVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SoundsMessage'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SNAPSHOTOVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SNAPSHOTOVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_SnapshotOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SNAPSHOTERROR"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SNAPSHOTERROR"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030 #GameUI_Disconnect_SnapshotError'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_RELIABLEOVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_RELIABLEOVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_ReliableOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADDELTATICK"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADDELTATICK"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030%#GameUI_Disconnect_BadClientDeltaTick'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NOMORESPLITS"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NOMORESPLITS"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_NoMoreSplits'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TIMEDOUT"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_TIMEDOUT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_TimedOut'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECTED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DISCONNECTED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_Disconnected'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_LEAVINGSPLIT"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_LEAVINGSPLIT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_LeavingSplit'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DIFFERENTCLASSTABLES"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DIFFERENTCLASSTABLES"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\'#GameUI_Disconnect_DifferentClassTables'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADRELAYPASSWORD"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADRELAYPASSWORD"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030##GameUI_Disconnect_BadRelayPassword'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADSPECTATORPASSWORD"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BADSPECTATORPASSWORD"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\'#GameUI_Disconnect_BadSpectatorPassword'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVRESTRICTED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVRESTRICTED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_HLTVRestricted'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NOSPECTATORS"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NOSPECTATORS"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_NoSpectators'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVUNAVAILABLE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVUNAVAILABLE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_HLTVUnavailable'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVSTOP"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVSTOP"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_HLTVStop'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_KICKED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_KICKED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\031#GameUI_Disconnect_Kicked'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BANADDED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BANADDED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\033#GameUI_Disconnect_BanAdded'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_KICKBANADDED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_KICKBANADDED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_KickBanAdded'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVDIRECT"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_HLTVDIRECT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\035#GameUI_Disconnect_HLTVDirect'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PURESERVER_CLIENTEXTRA"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PURESERVER_CLIENTEXTRA"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030)#GameUI_Disconnect_PureServer_ClientExtra'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PURESERVER_MISMATCH"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PURESERVER_MISMATCH"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_PureServer_Mismatch'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_USERCMD"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_USERCMD"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\032#GameUI_Disconnect_UserCmd'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_REJECTED_BY_GAME"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_REJECTED_BY_GAME"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030!#GameUI_Disconnect_RejectedByGame'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_MESSAGE_PARSE_ERROR"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_MESSAGE_PARSE_ERROR"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_MessageParseError'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_INVALID_MESSAGE_ERROR"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_INVALID_MESSAGE_ERROR"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_InvalidMessageError'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BAD_SERVER_PASSWORD"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_BAD_SERVER_PASSWORD"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_BadServerPassword'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DIRECT_CONNECT_RESERVATION"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_DIRECT_CONNECT_RESERVATION"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030+#GameUI_Disconnect_DirectConnectReservation'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_CONNECTION_FAILURE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_CONNECTION_FAILURE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_ConnectionFailure'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NO_PEER_GROUP_HANDLERS"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NO_PEER_GROUP_HANDLERS"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030&#GameUI_Disconnect_NoPeerGroupHandlers'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_RECONNECTION"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_RECONNECTION"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\037#GameUI_Disconnect_Reconnection'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_CONNECTION_CLOSING"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_CONNECTION_CLOSING"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030$#GameUI_Disconnect_ConnectionClosing'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NO_GOTV_RELAYS_AVAILABLE"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_NO_GOTV_RELAYS_AVAILABLE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030(#GameUI_Disconnect_NoGOTVRelaysAvailable'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SESSION_MIGRATED"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SESSION_MIGRATED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_SessionMigrated'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_VERYLARGETRANSFEROVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_VERYLARGETRANSFEROVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030,#GameUI_Disconnect_VeryLargeTransferOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SENDNETOVERFLOW"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SENDNETOVERFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_SendNetOverflow'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PLAYER_REMOVED_FROM_HOST_SESSION"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_PLAYER_REMOVED_FROM_HOST_SESSION"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030/#GameUI_Disconnect_PlayerRemovedFromHostSession'))
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SERVER_DOS"].has_options = True
_ENETWORKDISCONNECTIONREASON.values_by_name["NETWORK_DISCONNECT_SERVER_DOS"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\242\324\030\"#GameUI_Disconnect_TooManyCommands'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: demoparser/protobufs/gcsystemmsgs.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='demoparser/protobufs/gcsystemmsgs.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n\'demoparser/protobufs/gcsystemmsgs.proto\"S\n\x1c\x43MsgGCHVacVerificationChange\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\r\n\x05\x61ppid\x18\x02 \x01(\r\x12\x13\n\x0bis_verified\x18\x03 \x01(\x08\"\x80\x01\n\x1f\x43MsgGCHAccountPhoneNumberChange\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\r\n\x05\x61ppid\x18\x02 \x01(\r\x12\x10\n\x08phone_id\x18\x03 \x01(\x04\x12\x13\n\x0bis_verified\x18\x04 \x01(\x08\x12\x16\n\x0eis_identifying\x18\x05 \x01(\x08\"j\n\x18\x43MsgGCHInviteUserToLobby\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\r\n\x05\x61ppid\x18\x02 \x01(\r\x12\x17\n\x0fsteamid_invited\x18\x03 \x01(\x06\x12\x15\n\rsteamid_lobby\x18\x04 \x01(\x06*\xc7\x14\n\x0c\x45GCSystemMsg\x12\x13\n\x0fk_EGCMsgInvalid\x10\x00\x12\x11\n\rk_EGCMsgMulti\x10\x01\x12\x18\n\x14k_EGCMsgGenericReply\x10\n\x12\x16\n\x12k_EGCMsgSystemBase\x10\x32\x12\x1e\n\x1ak_EGCMsgAchievementAwarded\x10\x33\x12\x16\n\x12k_EGCMsgConCommand\x10\x34\x12\x18\n\x14k_EGCMsgStartPlaying\x10\x35\x12\x17\n\x13k_EGCMsgStopPlaying\x10\x36\x12\x1b\n\x17k_EGCMsgStartGameserver\x10\x37\x12\x1a\n\x16k_EGCMsgStopGameserver\x10\x38\x12\x15\n\x11k_EGCMsgWGRequest\x10\x39\x12\x16\n\x12k_EGCMsgWGResponse\x10:\x12\"\n\x1ek_EGCMsgGetUserGameStatsSchema\x10;\x12*\n&k_EGCMsgGetUserGameStatsSchemaResponse\x10<\x12\"\n\x1ek_EGCMsgGetUserStatsDEPRECATED\x10=\x12 \n\x1ck_EGCMsgGetUserStatsResponse\x10>\x12\x1a\n\x16k_EGCMsgAppInfoUpdated\x10?\x12\x1b\n\x17k_EGCMsgValidateSession\x10@\x12#\n\x1fk_EGCMsgValidateSessionResponse\x10\x41\x12\"\n\x1ek_EGCMsgLookupAccountFromInput\x10\x42\x12\x1b\n\x17k_EGCMsgSendHTTPRequest\x10\x43\x12#\n\x1fk_EGCMsgSendHTTPRequestResponse\x10\x44\x12\x18\n\x14k_EGCMsgPreTestSetup\x10\x45\x12\x1f\n\x1bk_EGCMsgRecordSupportAction\x10\x46\x12(\n$k_EGCMsgGetAccountDetails_DEPRECATED\x10G\x12\"\n\x1ek_EGCMsgReceiveInterAppMessage\x10I\x12\x18\n\x14k_EGCMsgFindAccounts\x10J\x12\x15\n\x11k_EGCMsgPostAlert\x10K\x12\x17\n\x13k_EGCMsgGetLicenses\x10L\x12\x18\n\x14k_EGCMsgGetUserStats\x10M\x12\x17\n\x13k_EGCMsgGetCommands\x10N\x12\x1f\n\x1bk_EGCMsgGetCommandsResponse\x10O\x12\x1a\n\x16k_EGCMsgAddFreeLicense\x10P\x12\"\n\x1ek_EGCMsgAddFreeLicenseResponse\x10Q\x12\x19\n\x15k_EGCMsgGetIPLocation\x10R\x12!\n\x1dk_EGCMsgGetIPLocationResponse\x10S\x12\x1d\n\x19k_EGCMsgSystemStatsSchema\x10T\x12\x1a\n\x16k_EGCMsgGetSystemStats\x10U\x12\"\n\x1ek_EGCMsgGetSystemStatsResponse\x10V\x12\x15\n\x11k_EGCMsgSendEmail\x10W\x12\x1d\n\x19k_EGCMsgSendEmailResponse\x10X\x12\x1c\n\x18k_EGCMsgGetEmailTemplate\x10Y\x12$\n k_EGCMsgGetEmailTemplateResponse\x10Z\x12\x1a\n\x16k_EGCMsgGrantGuestPass\x10[\x12\"\n\x1ek_EGCMsgGrantGuestPassResponse\x10\\\x12\x1d\n\x19k_EGCMsgGetAccountDetails\x10]\x12%\n!k_EGCMsgGetAccountDetailsResponse\x10^\x12\x1b\n\x17k_EGCMsgGetPersonaNames\x10_\x12#\n\x1fk_EGCMsgGetPersonaNamesResponse\x10`\x12\x18\n\x14k_EGCMsgMultiplexMsg\x10\x61\x12 \n\x1ck_EGCMsgMultiplexMsgResponse\x10\x62\x12$\n k_EGCMsgWebAPIRegisterInterfaces\x10\x65\x12\x1c\n\x18k_EGCMsgWebAPIJobRequest\x10\x66\x12(\n$k_EGCMsgWebAPIJobRequestHttpResponse\x10h\x12+\n\'k_EGCMsgWebAPIJobRequestForwardResponse\x10i\x12\x19\n\x14k_EGCMsgMemCachedGet\x10\xc8\x01\x12!\n\x1ck_EGCMsgMemCachedGetResponse\x10\xc9\x01\x12\x19\n\x14k_EGCMsgMemCachedSet\x10\xca\x01\x12\x1c\n\x17k_EGCMsgMemCachedDelete\x10\xcb\x01\x12\x1b\n\x16k_EGCMsgMemCachedStats\x10\xcc\x01\x12#\n\x1ek_EGCMsgMemCachedStatsResponse\x10\xcd\x01\x12\x1f\n\x1ak_EGCMsgMasterSetDirectory\x10\xdc\x01\x12\'\n\"k_EGCMsgMasterSetDirectoryResponse\x10\xdd\x01\x12#\n\x1ek_EGCMsgMasterSetWebAPIRouting\x10\xde\x01\x12+\n&k_EGCMsgMasterSetWebAPIRoutingResponse\x10\xdf\x01\x12&\n!k_EGCMsgMasterSetClientMsgRouting\x10\xe0\x01\x12.\n)k_EGCMsgMasterSetClientMsgRoutingResponse\x10\xe1\x01\x12\x17\n\x12k_EGCMsgSetOptions\x10\xe2\x01\x12\x1f\n\x1ak_EGCMsgSetOptionsResponse\x10\xe3\x01\x12\x18\n\x13k_EGCMsgSystemBase2\x10\xf4\x03\x12#\n\x1ek_EGCMsgGetPurchaseTrustStatus\x10\xf5\x03\x12+\n&k_EGCMsgGetPurchaseTrustStatusResponse\x10\xf6\x03\x12\x1a\n\x15k_EGCMsgUpdateSession\x10\xf7\x03\x12%\n k_EGCMsgGCAccountVacStatusChange\x10\xf8\x03\x12\x1c\n\x17k_EGCMsgCheckFriendship\x10\xf9\x03\x12$\n\x1fk_EGCMsgCheckFriendshipResponse\x10\xfa\x03\x12\"\n\x1dk_EGCMsgGetPartnerAccountLink\x10\xfb\x03\x12*\n%k_EGCMsgGetPartnerAccountLinkResponse\x10\xfc\x03\x12\"\n\x1dk_EGCMsgVacVerificationChange\x10\x86\x04\x12%\n k_EGCMsgAccountPhoneNumberChange\x10\x87\x04\x12\x1e\n\x19k_EGCMsgInviteUserToLobby\x10\x8b\x04*\xef\x01\n\x06\x45SOMsg\x12\x13\n\x0fk_ESOMsg_Create\x10\x15\x12\x13\n\x0fk_ESOMsg_Update\x10\x16\x12\x14\n\x10k_ESOMsg_Destroy\x10\x17\x12\x1c\n\x18k_ESOMsg_CacheSubscribed\x10\x18\x12\x1e\n\x1ak_ESOMsg_CacheUnsubscribed\x10\x19\x12\x1b\n\x17k_ESOMsg_UpdateMultiple\x10\x1a\x12#\n\x1fk_ESOMsg_CacheSubscriptionCheck\x10\x1b\x12%\n!k_ESOMsg_CacheSubscriptionRefresh\x10\x1c*\xd9\x02\n\x10\x45GCBaseClientMsg\x12\x1a\n\x15k_EMsgGCClientWelcome\x10\xa4\x1f\x12\x1a\n\x15k_EMsgGCServerWelcome\x10\xa5\x1f\x12\x18\n\x13k_EMsgGCClientHello\x10\xa6\x1f\x12\x18\n\x13k_EMsgGCServerHello\x10\xa7\x1f\x12#\n\x1ek_EMsgGCClientConnectionStatus\x10\xa9\x1f\x12#\n\x1ek_EMsgGCServerConnectionStatus\x10\xaa\x1f\x12\x1f\n\x1ak_EMsgGCClientHelloPartner\x10\xab\x1f\x12\x1a\n\x15k_EMsgGCClientHelloPW\x10\xac\x1f\x12\x1a\n\x15k_EMsgGCClientHelloR2\x10\xad\x1f\x12\x1a\n\x15k_EMsgGCClientHelloR3\x10\xae\x1f\x12\x1a\n\x15k_EMsgGCClientHelloR4\x10\xaf\x1f*\x89\x02\n\nEGCToGCMsg\x12\x1a\n\x15k_EGCToGCMsgMasterAck\x10\x96\x01\x12\"\n\x1dk_EGCToGCMsgMasterAckResponse\x10\x97\x01\x12\x17\n\x12k_EGCToGCMsgRouted\x10\x98\x01\x12\x1c\n\x17k_EGCToGCMsgRoutedReply\x10\x99\x01\x12\x1a\n\x15k_EMsgUpdateSessionIP\x10\x9a\x01\x12\x1b\n\x16k_EMsgRequestSessionIP\x10\x9b\x01\x12#\n\x1ek_EMsgRequestSessionIPResponse\x10\x9c\x01\x12&\n!k_EGCToGCMsgMasterStartupComplete\x10\x9d\x01\x42\x05H\x01\x80\x01\x00')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_EGCSYSTEMMSG = _descriptor.EnumDescriptor(
  name='EGCSystemMsg',
  full_name='EGCSystemMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgInvalid', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMulti', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGenericReply', index=2, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSystemBase', index=3, number=50,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgAchievementAwarded', index=4, number=51,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgConCommand', index=5, number=52,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgStartPlaying', index=6, number=53,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgStopPlaying', index=7, number=54,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgStartGameserver', index=8, number=55,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgStopGameserver', index=9, number=56,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWGRequest', index=10, number=57,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWGResponse', index=11, number=58,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetUserGameStatsSchema', index=12, number=59,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetUserGameStatsSchemaResponse', index=13, number=60,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetUserStatsDEPRECATED', index=14, number=61,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetUserStatsResponse', index=15, number=62,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgAppInfoUpdated', index=16, number=63,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgValidateSession', index=17, number=64,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgValidateSessionResponse', index=18, number=65,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgLookupAccountFromInput', index=19, number=66,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSendHTTPRequest', index=20, number=67,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSendHTTPRequestResponse', index=21, number=68,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgPreTestSetup', index=22, number=69,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgRecordSupportAction', index=23, number=70,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetAccountDetails_DEPRECATED', index=24, number=71,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgReceiveInterAppMessage', index=25, number=73,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgFindAccounts', index=26, number=74,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgPostAlert', index=27, number=75,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetLicenses', index=28, number=76,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetUserStats', index=29, number=77,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetCommands', index=30, number=78,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetCommandsResponse', index=31, number=79,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgAddFreeLicense', index=32, number=80,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgAddFreeLicenseResponse', index=33, number=81,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetIPLocation', index=34, number=82,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetIPLocationResponse', index=35, number=83,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSystemStatsSchema', index=36, number=84,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetSystemStats', index=37, number=85,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetSystemStatsResponse', index=38, number=86,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSendEmail', index=39, number=87,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSendEmailResponse', index=40, number=88,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetEmailTemplate', index=41, number=89,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetEmailTemplateResponse', index=42, number=90,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGrantGuestPass', index=43, number=91,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGrantGuestPassResponse', index=44, number=92,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetAccountDetails', index=45, number=93,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetAccountDetailsResponse', index=46, number=94,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPersonaNames', index=47, number=95,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPersonaNamesResponse', index=48, number=96,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMultiplexMsg', index=49, number=97,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMultiplexMsgResponse', index=50, number=98,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWebAPIRegisterInterfaces', index=51, number=101,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWebAPIJobRequest', index=52, number=102,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWebAPIJobRequestHttpResponse', index=53, number=104,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgWebAPIJobRequestForwardResponse', index=54, number=105,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedGet', index=55, number=200,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedGetResponse', index=56, number=201,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedSet', index=57, number=202,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedDelete', index=58, number=203,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedStats', index=59, number=204,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMemCachedStatsResponse', index=60, number=205,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetDirectory', index=61, number=220,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetDirectoryResponse', index=62, number=221,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetWebAPIRouting', index=63, number=222,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetWebAPIRoutingResponse', index=64, number=223,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetClientMsgRouting', index=65, number=224,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgMasterSetClientMsgRoutingResponse', index=66, number=225,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSetOptions', index=67, number=226,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSetOptionsResponse', index=68, number=227,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgSystemBase2', index=69, number=500,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPurchaseTrustStatus', index=70, number=501,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPurchaseTrustStatusResponse', index=71, number=502,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgUpdateSession', index=72, number=503,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGCAccountVacStatusChange', index=73, number=504,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgCheckFriendship', index=74, number=505,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgCheckFriendshipResponse', index=75, number=506,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPartnerAccountLink', index=76, number=507,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgGetPartnerAccountLinkResponse', index=77, number=508,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgVacVerificationChange', index=78, number=518,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgAccountPhoneNumberChange', index=79, number=519,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCMsgInviteUserToLobby', index=80, number=523,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=368,
  serialized_end=2999,
)
_sym_db.RegisterEnumDescriptor(_EGCSYSTEMMSG)

EGCSystemMsg = enum_type_wrapper.EnumTypeWrapper(_EGCSYSTEMMSG)
_ESOMSG = _descriptor.EnumDescriptor(
  name='ESOMsg',
  full_name='ESOMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Create', index=0, number=21,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Update', index=1, number=22,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Destroy', index=2, number=23,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscribed', index=3, number=24,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheUnsubscribed', index=4, number=25,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_UpdateMultiple', index=5, number=26,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscriptionCheck', index=6, number=27,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscriptionRefresh', index=7, number=28,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3002,
  serialized_end=3241,
)
_sym_db.RegisterEnumDescriptor(_ESOMSG)

ESOMsg = enum_type_wrapper.EnumTypeWrapper(_ESOMSG)
_EGCBASECLIENTMSG = _descriptor.EnumDescriptor(
  name='EGCBaseClientMsg',
  full_name='EGCBaseClientMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientWelcome', index=0, number=4004,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerWelcome', index=1, number=4005,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHello', index=2, number=4006,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerHello', index=3, number=4007,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientConnectionStatus', index=4, number=4009,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerConnectionStatus', index=5, number=4010,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHelloPartner', index=6, number=4011,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHelloPW', index=7, number=4012,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHelloR2', index=8, number=4013,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHelloR3', index=9, number=4014,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHelloR4', index=10, number=4015,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3244,
  serialized_end=3589,
)
_sym_db.RegisterEnumDescriptor(_EGCBASECLIENTMSG)

EGCBaseClientMsg = enum_type_wrapper.EnumTypeWrapper(_EGCBASECLIENTMSG)
_EGCTOGCMSG = _descriptor.EnumDescriptor(
  name='EGCToGCMsg',
  full_name='EGCToGCMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_EGCToGCMsgMasterAck', index=0, number=150,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCToGCMsgMasterAckResponse', index=1, number=151,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCToGCMsgRouted', index=2, number=152,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCToGCMsgRoutedReply', index=3, number=153,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgUpdateSessionIP', index=4, number=154,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgRequestSessionIP', index=5, number=155,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgRequestSessionIPResponse', index=6, number=156,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EGCToGCMsgMasterStartupComplete', index=7, number=157,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3592,
  serialized_end=3857,
)
_sym_db.RegisterEnumDescriptor(_EGCTOGCMSG)

EGCToGCMsg = enum_type_wrapper.EnumTypeWrapper(_EGCTOGCMSG)
k_EGCMsgInvalid = 0
k_EGCMsgMulti = 1
k_EGCMsgGenericReply = 10
k_EGCMsgSystemBase = 50
k_EGCMsgAchievementAwarded = 51
k_EGCMsgConCommand = 52
k_EGCMsgStartPlaying = 53
k_EGCMsgStopPlaying = 54
k_EGCMsgStartGameserver = 55
k_EGCMsgStopGameserver = 56
k_EGCMsgWGRequest = 57
k_EGCMsgWGResponse = 58
k_EGCMsgGetUserGameStatsSchema = 59
k_EGCMsgGetUserGameStatsSchemaResponse = 60
k_EGCMsgGetUserStatsDEPRECATED = 61
k_EGCMsgGetUserStatsResponse = 62
k_EGCMsgAppInfoUpdated = 63
k_EGCMsgValidateSession = 64
k_EGCMsgValidateSessionResponse = 65
k_EGCMsgLookupAccountFromInput = 66
k_EGCMsgSendHTTPRequest = 67
k_EGCMsgSendHTTPRequestResponse = 68
k_EGCMsgPreTestSetup = 69
k_EGCMsgRecordSupportAction = 70
k_EGCMsgGetAccountDetails_DEPRECATED = 71
k_EGCMsgReceiveInterAppMessage = 73
k_EGCMsgFindAccounts = 74
k_EGCMsgPostAlert = 75
k_EGCMsgGetLicenses = 76
k_EGCMsgGetUserStats = 77
k_EGCMsgGetCommands = 78
k_EGCMsgGetCommandsResponse = 79
k_EGCMsgAddFreeLicense = 80
k_EGCMsgAddFreeLicenseResponse = 81
k_EGCMsgGetIPLocation = 82
k_EGCMsgGetIPLocationResponse = 83
k_EGCMsgSystemStatsSchema = 84
k_EGCMsgGetSystemStats = 85
k_EGCMsgGetSystemStatsResponse = 86
k_EGCMsgSendEmail = 87
k_EGCMsgSendEmailResponse = 88
k_EGCMsgGetEmailTemplate = 89
k_EGCMsgGetEmailTemplateResponse = 90
k_EGCMsgGrantGuestPass = 91
k_EGCMsgGrantGuestPassResponse = 92
k_EGCMsgGetAccountDetails = 93
k_EGCMsgGetAccountDetailsResponse = 94
k_EGCMsgGetPersonaNames = 95
k_EGCMsgGetPersonaNamesResponse = 96
k_EGCMsgMultiplexMsg = 97
k_EGCMsgMultiplexMsgResponse = 98
k_EGCMsgWebAPIRegisterInterfaces = 101
k_EGCMsgWebAPIJobRequest = 102
k_EGCMsgWebAPIJobRequestHttpResponse = 104
k_EGCMsgWebAPIJobRequestForwardResponse = 105
k_EGCMsgMemCachedGet = 200
k_EGCMsgMemCachedGetResponse = 201
k_EGCMsgMemCachedSet = 202
k_EGCMsgMemCachedDelete = 203
k_EGCMsgMemCachedStats = 204
k_EGCMsgMemCachedStatsResponse = 205
k_EGCMsgMasterSetDirectory = 220
k_EGCMsgMasterSetDirectoryResponse = 221
k_EGCMsgMasterSetWebAPIRouting = 222
k_EGCMsgMasterSetWebAPIRoutingResponse = 223
k_EGCMsgMasterSetClientMsgRouting = 224
k_EGCMsgMasterSetClientMsgRoutingResponse = 225
k_EGCMsgSetOptions = 226
k_EGCMsgSetOptionsResponse = 227
k_EGCMsgSystemBase2 = 500
k_EGCMsgGetPurchaseTrustStatus = 501
k_EGCMsgGetPurchaseTrustStatusResponse = 502
k_EGCMsgUpdateSession = 503
k_EGCMsgGCAccountVacStatusChange = 504
k_EGCMsgCheckFriendship = 505
k_EGCMsgCheckFriendshipResponse = 506
k_EGCMsgGetPartnerAccountLink = 507
k_EGCMsgGetPartnerAccountLinkResponse = 508
k_EGCMsgVacVerificationChange = 518
k_EGCMsgAccountPhoneNumberChange = 519
k_EGCMsgInviteUserToLobby = 523
k_ESOMsg_Create = 21
k_ESOMsg_Update = 22
k_ESOMsg_Destroy = 23
k_ESOMsg_CacheSubscribed = 24
k_ESOMsg_CacheUnsubscribed = 25
k_ESOMsg_UpdateMultiple = 26
k_ESOMsg_CacheSubscriptionCheck = 27
k_ESOMsg_CacheSubscriptionRefresh = 28
k_EMsgGCClientWelcome = 4004
k_EMsgGCServerWelcome = 4005
k_EMsgGCClientHello = 4006
k_EMsgGCServerHello = 4007
k_EMsgGCClientConnectionStatus = 4009
k_EMsgGCServerConnectionStatus = 4010
k_EMsgGCClientHelloPartner = 4011
k_EMsgGCClientHelloPW = 4012
k_EMsgGCClientHelloR2 = 4013
k_EMsgGCClientHelloR3 = 4014
k_EMsgGCClientHelloR4 = 4015
k_EGCToGCMsgMasterAck = 150
k_EGCToGCMsgMasterAckResponse = 151
k_EGCToGCMsgRouted = 152
k_EGCToGCMsgRoutedReply = 153
k_EMsgUpdateSessionIP = 154
k_EMsgRequestSessionIP = 155
k_EMsgRequestSessionIPResponse = 156
k_EGCToGCMsgMasterStartupComplete = 157



_CMSGGCHVACVERIFICATIONCHANGE = _descriptor.Descriptor(
  name='CMsgGCHVacVerificationChange',
  full_name='CMsgGCHVacVerificationChange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgGCHVacVerificationChange.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgGCHVacVerificationChange.appid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_verified', full_name='CMsgGCHVacVerificationChange.is_verified', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=126,
)


_CMSGGCHACCOUNTPHONENUMBERCHANGE = _descriptor.Descriptor(
  name='CMsgGCHAccountPhoneNumberChange',
  full_name='CMsgGCHAccountPhoneNumberChange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgGCHAccountPhoneNumberChange.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgGCHAccountPhoneNumberChange.appid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phone_id', full_name='CMsgGCHAccountPhoneNumberChange.phone_id', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_verified', full_name='CMsgGCHAccountPhoneNumberChange.is_verified', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_identifying', full_name='CMsgGCHAccountPhoneNumberChange.is_identifying', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=129,
  serialized_end=257,
)


_CMSGGCHINVITEUSERTOLOBBY = _descriptor.Descriptor(
  name='CMsgGCHInviteUserToLobby',
  full_name='CMsgGCHInviteUserToLobby',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CMsgGCHInviteUserToLobby.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CMsgGCHInviteUserToLobby.appid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamid_invited', full_name='CMsgGCHInviteUserToLobby.steamid_invited', index=2,
      number=3, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='steamid_lobby', full_name='CMsgGCHInviteUserToLobby.steamid_lobby', index=3,
      number=4, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=259,
  serialized_end=365,
)

DESCRIPTOR.message_types_by_name['CMsgGCHVacVerificationChange'] = _CMSGGCHVACVERIFICATIONCHANGE
DESCRIPTOR.message_types_by_name['CMsgGCHAccountPhoneNumberChange'] = _CMSGGCHACCOUNTPHONENUMBERCHANGE
DESCRIPTOR.message_types_by_name['CMsgGCHInviteUserToLobby'] = _CMSGGCHINVITEUSERTOLOBBY
DESCRIPTOR.enum_types_by_name['EGCSystemMsg'] = _EGCSYSTEMMSG
DESCRIPTOR.enum_types_by_name['ESOMsg'] = _ESOMSG
DESCRIPTOR.enum_types_by_name['EGCBaseClientMsg'] = _EGCBASECLIENTMSG
DESCRIPTOR.enum_types_by_name['EGCToGCMsg'] = _EGCTOGCMSG

CMsgGCHVacVerificationChange = _reflection.GeneratedProtocolMessageType('CMsgGCHVacVerificationChange', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGCHVACVERIFICATIONCHANGE,
  __module__ = 'demoparser.protobufs.gcsystemmsgs_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGCHVacVerificationChange)
  ))
_sym_db.RegisterMessage(CMsgGCHVacVerificationChange)

CMsgGCHAccountPhoneNumberChange = _reflection.GeneratedProtocolMessageType('CMsgGCHAccountPhoneNumberChange', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGCHACCOUNTPHONENUMBERCHANGE,
  __module__ = 'demoparser.protobufs.gcsystemmsgs_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGCHAccountPhoneNumberChange)
  ))
_sym_db.RegisterMessage(CMsgGCHAccountPhoneNumberChange)

CMsgGCHInviteUserToLobby = _reflection.GeneratedProtocolMessageType('CMsgGCHInviteUserToLobby', (_message.Message,), dict(
  DESCRIPTOR = _CMSGGCHINVITEUSERTOLOBBY,
  __module__ = 'demoparser.protobufs.gcsystemmsgs_pb2'
  # @@protoc_insertion_point(class_scope:CMsgGCHInviteUserToLobby)
  ))
_sym_db.RegisterMessage(CMsgGCHInviteUserToLobby)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('H\001\200\001\000'))
# @@protoc_insertion_point(module_scope)



cdef class Bitbuffer:
    cdef unsigned int num_bytes
    cdef unsigned int * data
    cdef unsigned int index
    cdef unsigned int length
    cdef unsigned int bits_avail
    cdef unsigned int in_buf_word
    cdef bytes orig_data

    cpdef unsigned int next_dword(self)
    cpdef unsigned char read_bit(self)
    cpdef unsigned int read_uint_bits(self, unsigned int bits)
    cpdef int read_sint_bits(self, unsigned int bits)
    cpdef unsigned int read_var_int(self)
    cpdef str read_string(self, int length=*)
    cpdef float read_bit_normal(self)
    cpdef float read_bit_coord(self)
    cpdef float read_bit_cell_coord(self, unsigned int bits,
            unsigned int coord_type)
    cpdef bytes read_user_data(self, unsigned int bits)
